'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  const callback = this.async();
  load(this).then(r => callback(null, r), e => callback(e, null));
};

var _asyncChildProcess = require('async-child-process');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _loaderUtils = require('loader-utils');

var _loaderUtils2 = _interopRequireDefault(_loaderUtils);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _arraybufferToString = require('arraybuffer-to-string');

var _arraybufferToString2 = _interopRequireDefault(_arraybufferToString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const findSrcDir = (() => {
  var _ref = _asyncToGenerator(function* (childPath) {
    let candidate = childPath;

    while (candidate !== _path2.default.parse(candidate).root) {
      const maybeCargoFile = _path2.default.join(candidate, 'Cargo.toml');
      if (yield _fsExtra2.default.pathExists(maybeCargoFile)) {
        return candidate;
      }
      candidate = _path2.default.dirname(candidate);
    }

    return null;
  });

  return function findSrcDir(_x) {
    return _ref.apply(this, arguments);
  };
})();

const readBinCrateName = (() => {
  var _ref2 = _asyncToGenerator(function* (srcDir) {
    const result = yield (0, _asyncChildProcess.execAsync)('cargo read-manifest', { cwd: srcDir });
    const data = JSON.parse(result.stdout);

    let binCrate = data.targets.find(function (t) {
      return t.kind.indexOf('bin') >= 0;
    });

    if (binCrate) {
      return binCrate.name;
    } else {
      throw new Error(`No bin crate found in ${srcDir}`);
    }
  });

  return function readBinCrateName(_x2) {
    return _ref2.apply(this, arguments);
  };
})();

const DEFAULT_OPTIONS = {
  release: false,
  gc: false,
  target: 'wasm32-unknown-unknown',
  cargoWeb: false,
  name: undefined,
  regExp: undefined
};

const loadCargoWeb = (() => {
  var _ref3 = _asyncToGenerator(function* (self, opts, srcDir) {
    const release = opts.release;
    const target = opts.target;
    const name = opts.name;

    const crateName = yield readBinCrateName(srcDir);
    const jsFileName = crateName + '.js';
    const wasmFileName = crateName + '.wasm';

    const cmd = `cargo web build --target-webasm${release ? ' --release' : ''}`;
    yield (0, _asyncChildProcess.execAsync)(cmd, { cwd: srcDir });

    const outDir = _path2.default.join(srcDir, 'target', target, release ? 'release' : 'debug');

    const jsData = yield _fsExtra2.default.readFile(_path2.default.join(outDir, jsFileName));
    const wasmData = yield _fsExtra2.default.readFile(_path2.default.join(outDir, wasmFileName));

    const context = opts.context || self.rootContext || self.options && self.options.context;
    const wasmOutFileName = _loaderUtils2.default.interpolateName(self, name, {
      context, content: wasmData, regExp: opts.regExp
    });

    self.emitFile(wasmOutFileName, wasmData);

    // Ugly way to do replaceAll
    return (0, _arraybufferToString2.default)(jsData).split(JSON.stringify(wasmFileName)).join(`__webpack_public_path__ + ${JSON.stringify(wasmOutFileName)}`);
  });

  return function loadCargoWeb(_x3, _x4, _x5) {
    return _ref3.apply(this, arguments);
  };
})();

const loadRaw = (() => {
  var _ref4 = _asyncToGenerator(function* (opts, srcDir) {
    const release = opts.release;
    const gc = opts.gc;
    const target = opts.target;
    const cmd = `cargo build --message-format=json --target=${target}${release ? ' --release' : ''}`;

    const result = yield (0, _asyncChildProcess.execAsync)(cmd, { cwd: srcDir });

    let wasmFile;
    for (let line of result.stdout.split(_os2.default.EOL)) {
      if (/^\s*$/.test(line)) {
        continue;
      }
      const data = JSON.parse(line);
      if (data.hasOwnProperty('filenames')) {
        wasmFile = data['filenames'].find(function (p) {
          return p.endsWith('.wasm');
        });
        if (wasmFile) {
          break;
        }
      }
    }

    if (!wasmFile) {
      throw new Error('No wasm file produced as build output', null);
    }

    if (gc) {
      let gcWasmFile = wasmFile.substr(0, wasmFile.length - '.wasm'.length) + '.gc.wasm';
      yield (0, _asyncChildProcess.execAsync)(`wasm-gc ${wasmFile} ${gcWasmFile}`);
      wasmFile = gcWasmFile;
    }

    return yield _fsExtra2.default.readFile(wasmFile);
  });

  return function loadRaw(_x6, _x7) {
    return _ref4.apply(this, arguments);
  };
})();

const load = (() => {
  var _ref5 = _asyncToGenerator(function* (self) {
    const srcDir = yield findSrcDir(self.resourcePath);
    if (!srcDir) {
      throw new Error('No Cargo.toml file found in any parent directory.');
    }

    const opts = Object.assign({}, DEFAULT_OPTIONS, _loaderUtils2.default.getOptions(self));
    const cargoWeb = opts.cargoWeb;

    if (cargoWeb) {
      return yield loadCargoWeb(self, opts, srcDir);
    } else {
      return yield loadRaw(opts, srcDir);
    }
  });

  return function load(_x8) {
    return _ref5.apply(this, arguments);
  };
})();

;